def intelligent_mail ( code ):
	return code
def binary( code ):
	'''
	Convert a list of code indices to a binary string.
	'''
	return ''.join( [encoding[x] for x in code] )

def decode( code, verbose = False ):
	'''
	Decode a coded string given as a list of code indices. For testing purposes.
	'''
	decode = decode_A
	nextdecode = decode
	for c in code:
		d = decode[c]
		decode = nextdecode
		if d == '[START A]' or d == '[Code A]':
			nextdecode = decode = decode_A
		elif d == '[START B]' or d == '[Code B]':
			nextdecode = decode = decode_B
		elif d == '[START C]' or d == '[Code C]':
			nextdecode = decode = decode_C
		elif d == '[SHIFT A]':
			nextdecode = decode
			decode = decode_A
		elif d == '[SHIFT B]':
			nextdecode = decode
			decode = decode_B
		elif not verbose:
			yield d
		if verbose:
			yield d

def checksum( code ):
	'''
	Calculate the checksum for a coded string given as a list of code indices.
	'''
	sum = code[0]
	for i in range( 1, len( code ) ):
		sum += i * code[i]
	return sum % 103

def encode( input ):
	'''
	Optimally encodes an input in code 128, complete with checksum and stop. Returns code as a binary string or None if no encoding can be found.
	'''
	encoded = bestcode( input )
	if encoded is None:
		return None
	return binary( encoded + (checksum( encoded ))

if __name__ == '__main__':
	import sys
	encoded = bestcode( sys.argv[1] )
	print sys.argv[1]
	print 'encoded:', encoded
	print 'decoded:', [x for x in decode( encoded, verbose = True )]
	print 'checksum:', checksum( encoded )
	print 'binary:', encode( sys.argv[1] )

